/**
News Feed
Author: Milan Skipala

Sketch loads RSS feed content and displays headlines of articles
in random order. Implementation supports RSS feeds by reuters.

*/
import java.util.Arrays;
import java.util.HashSet;
import geomerative.*;

String feedURL = "http://feeds.reuters.com/news/artsculture";

XML xml;
PFont font;
RFont rfont;
String[] titles;
String textToPrint;
String[] lines;
int[] fontHeights;
RPoint[][] fontPoints;
RPoint[][] randomPoints;
float phase;
int animationFrames;
boolean animating;
int frames;
float opacity;
HashSet<Integer> alreadyDisplayed;

void setup() {
  size(1280, 600);
  frameRate(30);
  RG.init(this);
  RCommand.setSegmentLength(1);
  RCommand.setSegmentator(RCommand.UNIFORMLENGTH);

  //load news' headlines from feed 
  xml = loadXML(feedURL);
  XML[] children = xml.getChild("channel").getChildren("item");
  font = createFont("FreeSans.ttf", 150);
  rfont = new RFont("FreeSans.ttf", 150, RFont.LEFT);
  titles = new String[children.length];

  for (int i = 0; i < children.length; i++) {
    titles[i] = children[i].getChild("title").getContent();
    titles[i] = titles[i].toUpperCase();
  }
  alreadyDisplayed = new HashSet<Integer>();
  
  loadNewTitle();

  phase = 0;
  animationFrames = 0;
  animating = false;
  frames = 0;
  opacity = 255;
}

void prepareText() {
  //Split textToPrint into lines (1 line = all words of length
  //at most 3 followed by 1 word longer than 3.
  lines = new String[textToPrint.length()/2]; //worst case = "a a a..." - spaces will not be printed
  int numberOfLines = 0;
  int start = 0;
  int currentPosition = 0;
  while (currentPosition < textToPrint.length ()) {
    int lengthOfLastWord = 0;
    while (currentPosition < textToPrint.length ()) {
      if (textToPrint.charAt(currentPosition) == ' ' && lengthOfLastWord > 3) {
        break;
      }
      if (textToPrint.charAt(currentPosition) == ' ') {
        lengthOfLastWord = 0;
      }
      currentPosition++;
      lengthOfLastWord++;
    }
    lines[numberOfLines] = textToPrint.substring(start, currentPosition);
    numberOfLines++;
    currentPosition++;
    start = currentPosition;
  }

  lines = Arrays.copyOf(lines, numberOfLines);

  //Now we have number of lines so we can determine the height
  //of the characters. How to do it?
  // - start with some small font size, compute textWidth
  // - repeat while textWidth < width-borders
  fontHeights = new int[lines.length];
  int sum = 0;
  for (int i = 0; i < lines.length; i++) {
    int fontSize = 10;
    while (fontSize < height) {   
      textFont(font, fontSize);
      if (textWidth(lines[i]) < 1.05*width && textWidth(lines[i]) > 0.95*width)
        break;
      fontSize++;
    }
    fontHeights[i] = fontSize;
    sum += fontHeights[i];
  }

  //Height can be now larger than the viewport height. We will correct it if needed.
  if (sum > height) {
    float factor = sum/( height - 10.0 );
    factor = 1/factor;
    for (int i = 0; i < lines.length; i++) {
      fontHeights[i] = (int)(fontHeights[i] * factor);
    }
  }
}

void preparePoints() {
  // get the fontPoints on font outline
  fontPoints = new RPoint[lines.length][];
  randomPoints = new RPoint[lines.length][];
  for (int i = 0; i < lines.length; i++) {
    rfont = new RFont("FreeSans.ttf", fontHeights[i], RFont.CENTER);
    RGroup grp;
    grp = rfont.toGroup(lines[i]);
    grp = grp.toPolygonGroup();
    fontPoints[i] = grp.getPoints();
    randomPoints[i] = new RPoint[fontPoints[i].length];
    for (int j = 0; j < randomPoints[i].length; j++) {
      randomPoints[i][j] = new RPoint(random(width), random(fontHeights[i]));
    }
  }
}

void loadNewTitle() {
  int index = int(random(titles.length));
  while (alreadyDisplayed.contains(index)) {
    index = int(random(titles.length));
  }
  alreadyDisplayed.add(index);
  if (alreadyDisplayed.size()==titles.length) {
    alreadyDisplayed.clear();
  }
  textToPrint = titles[index];
  prepareText();
  preparePoints();
}

void draw() {
  //1pt fontPoints form the text
  //mouse influences the morphing between readable text and random fontPoints
  //start in completely random phase, when the text is fully readable, animate
  //the "explosion" (back to random) and show next (random?) headline
  background(0);
  if (opacity <= 0) {
    opacity = 5;
  }
  if (opacity < 255 && animating == false) {
    opacity *= 2;
  }
  if (opacity > 255) {
    opacity = 255;
  }
  fill(0,210,255,opacity);
  textAlign(CENTER);
  
  noStroke();

  textFont(font, 150);
  for (int i = 0; i < lines.length; i++) {
    translate(0, fontHeights[i]);

    //This line is not needed:
    //textFont(font, fontHeights[i]);
    
    float diameter = 2;
    for (int j = 0; j < fontPoints[i].length; j++ ) {
      ellipse(lerp(fontPoints[i][j].x+width/2, randomPoints[i][j].x, 1-phase), 
      lerp(fontPoints[i][j].y, randomPoints[i][j].y-fontHeights[i], 1-phase), 
      diameter, diameter);
    }
  }
  animationLogic();
}

void animationLogic() {
  int animationLength = 1;//seconds
  if (animating == false) {
    phase = frames/float(30*animationLength);
    
    if (phase > 0.99) {
      animating = true;    
    }
  } else {
    animationFrames++;
    if (animationFrames < 30) {
      phase = 1;
    } else {
      phase *= 0.85;
      opacity = lerp(0,255,lerp(-1.5,1,30.0/animationFrames));
    }
    if (phase < 0.02) {
      animating = false;
      frames = 0;
      animationFrames = 0;
      loadNewTitle();
    }
  }
  frames++;
}

